#!/bin/bash

# Creates the accumulated table for the benchmarks

if [ -n "$1" ]; then
	FOLDER="$1"
else
	FOLDER="benchmark-output"
fi

FILES=( )

for i in $(ls ${FOLDER}/*.tex); do
	FILES+=( ${i} )
done

MAX_ITERATIONS=500
NLOPT_TIMEOUT=10


OFS=$IFS
IFS=$'\n'

# Get the different k and store them in "Ks"
LINES=`cat ${FILES[0]} | grep -e "^[[:digit:]]"`

Ks=( )
Ds=( )

for i in $LINES; do
	k=`echo $i | cut -d '&' -f4 | tr -d ' '`
        Ks+=( $k )
        d=`echo $i | cut -d '&' -f2 | tr -d ' '`
        Ds+=( $d )
done

SORTED=`sort -u <<< "${Ks[*]}"`
SORTED_D=`sort -u <<< "${Ds[*]}"`

Ks=( )
Ds=( )

for i in "${SORTED}"; do
  Ks+=( $i )
done

for i in "${SORTED_D}"; do
  Ds+=( $i )
done

# Iterate over all the K and accumulate the values
for d in "${Ds[@]}"; do
for i in "${Ks[@]}"; do
	RESULT=`grep -he "^[[:digit:]]* & $d & [[:digit:]]* & $i &.*" $FOLDER/*.tex`
	
	NUM_SEP_EXP=0
	NUM_INT_EXP=0
	
	NUM_SEP_GJK=0
	NUM_INT_GJK=0
	NUM_INT_GJK_MAX=0
	
	NUM_GJK_TERM=0
	ITER_TERM_MIN=$MAX_ITERATIONS
	ITER_TERM_MAX=0
	ITER_TERM_SUM=0
	
	GJK_RUNTIME_MIN=1000000000
	GJK_RUNTIME_MAX=0
	GJK_RUNTIME_SUM=0

	for LINE in $RESULT; do
		EXP=`echo $LINE | cut -d '&' -f 5 | tr -d ' '`
		GJK=`echo $LINE | cut -d '&' -f 6 | tr -d ' '`

		ITER=`echo $LINE | cut -d '&' -f 7 | tr -d ' '`

		GJK_RUNTIME=`echo $LINE | cut -d '&' -f 8 | tr -d ' ' | sed "s/s//g" | sed "s/\\\\\\//g"`

		FACETS=`echo $LINE | cut -d '&' -f 3 | tr -d ' '`

		# If separation was expected
		if [ "$EXP" == "SEP" ]; then
			NUM_SEP_EXP=$(( $NUM_SEP_EXP + 1 ))

			if [ "$GJK" == "SEP" -a "$ITER" != "" ]; then
				NUM_SEP_GJK=$(( $NUM_SEP_GJK + 1 ))

				if [ "$ITER" -lt "$MAX_ITERATIONS" ]; then

					NUM_GJK_TERM=$(( $NUM_GJK_TERM + 1 ))

					if [ "$ITER_TERM_MIN" -gt "$ITER" ]; then
						ITER_TERM_MIN=$ITER
					fi

					if [ "$ITER_TERM_MAX" -lt "$ITER" ]; then
						ITER_TERM_MAX=$ITER
					fi
						
					ITER_TERM_SUM=$(( $ITER_TERM_SUM + $ITER ))

					if [ `echo "$GJK_RUNTIME_MIN > $GJK_RUNTIME" | bc -l` == "1" ]; then
						GJK_RUNTIME_MIN=$GJK_RUNTIME
					fi

					if [ `echo "$GJK_RUNTIME_MAX < $GJK_RUNTIME" | bc -l` == "1" ]; then
						GJK_RUNTIME_MAX=$GJK_RUNTIME
					fi

					GJK_RUNTIME_SUM=`bc <<< "( $GJK_RUNTIME_SUM + $GJK_RUNTIME )"`
				fi

			fi
		fi

		if [ "$EXP" == "INT" ]; then

			NUM_INT_EXP=$(( $NUM_INT_EXP + 1 ))

		fi

		if [ "$GJK" == "INT" -a "$ITER" != "" ]; then
			NUM_INT_GJK=$(( $NUM_INT_GJK + 1 ))

			if [ "$ITER" == "$MAX_ITERATIONS" ]; then
				NUM_INT_GJK_MAX=$(( $NUM_INT_GJK_MAX + 1 ))
			fi
		fi
	done

	GJK_PERCENTAGE=`bc <<< "scale=2; ( ($NUM_SEP_GJK / $NUM_SEP_EXP) * 100 )"`

	GJK_PERCENTAGE=`bc <<< "scale=0; ( $GJK_PERCENTAGE / 1)"`

	if [ $NUM_GJK_TERM -ne 0 ]; then
		INT_AVG=`bc <<< "scale=3; ( $ITER_TERM_SUM / $NUM_GJK_TERM )"`
	else
		INT_AVG="---"
	fi

	if [ "$NUM_GJK_TERM" -ne "0" ]; then
		GJK_RUNTIME_AVG=`bc <<< "scale=3; ( $GJK_RUNTIME_SUM / $NUM_GJK_TERM )"`
	else
		GJK_RUNTIME_AVG="---"
	fi

	GJK_RUNTIME_MIN=`bc <<< "scale=3; ( $GJK_RUNTIME_MIN / 1)"`
	GJK_RUNTIME_MAX=`bc <<< "scale=3; ( $GJK_RUNTIME_MAX / 1)"`

	if [ `echo "${GJK_RUNTIME_MIN} < 1" | bc -l` == "1" -a "${GJK_RUNTIME_MIN}" != "0" ]; then
		GJK_RUNTIME_MIN=0${GJK_RUNTIME_MIN}
	fi
		if [ `echo "${GJK_RUNTIME_MAX} < 1" | bc -l` == "1" -a "${GJK_RUNTIME_MAX}" != "0" ]; then
			GJK_RUNTIME_MAX=0${GJK_RUNTIME_MAX}
		fi

	if [ "$GJK_RUNTIME_AVG" != "---" ]; then
		if [ `echo "${GJK_RUNTIME_AVG} < 1" | bc -l` == "1" -a "${GJK_RUNTIME_AVG}" != "0" ]; then
			GJK_RUNTIME_AVG=0${GJK_RUNTIME_AVG}
		fi
	fi


	if [[ $FOLDER == *"IRON"* ]]; then
		ALGORITHM="GJK"
	fi

	if [[ $FOLDER == *"DIAMOND"* ]]; then
		ALGORITHM="DA"
	fi

	echo "$ALGORITHM & $d & $FACETS & $i & ${GJK_PERCENTAGE}\\% & $ITER_TERM_MIN & $ITER_TERM_MAX & $INT_AVG & ${GJK_RUNTIME_MIN}s & ${GJK_RUNTIME_MAX}s & ${GJK_RUNTIME_AVG}s \\\\"
done

echo "\\hline"

done

IFS=$OFS
